#include "FenServeur.h"

FenServeur::FenServeur()
{
    // Cration et disposition des widgets de la fentre
    etatServeur = new QLabel;
    boutonQuitter = new QPushButton(tr("Quitter"));
    connect(boutonQuitter, SIGNAL(clicked()), qApp, SLOT(quit()));

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(etatServeur);
    layout->addWidget(boutonQuitter);
    setLayout(layout);

    setWindowTitle(tr("ZeroChat - Serveur"));

    // Gestion du serveur
    serveur = new QTcpServer(this);
    if (!serveur->listen(QHostAddress::Any, 50885)) // Dmarrage du serveur sur toutes les IP disponibles et sur le port 50585
    {
        // Si le serveur n'a pas t dmarr correctement
        etatServeur->setText(tr("Le serveur n'a pas pu tre dmarr. Raison :<br />") + serveur->errorString());
    }
    else
    {
        // Si le serveur a t dmarr correctement
        etatServeur->setText(tr("Le serveur a t dmarr sur le port <strong>") + QString::number(serveur->serverPort()) + tr("</strong>.<br />Des clients peuvent maintenant se connecter."));
        connect(serveur, SIGNAL(newConnection()), this, SLOT(nouvelleConnexion()));
    }

    tailleMessage = 0;
}

void FenServeur::nouvelleConnexion()
{
    envoyerATous(tr("<em>Un nouveau client vient de se connecter</em>"));

    QTcpSocket *nouveauClient = serveur->nextPendingConnection();
    clients << nouveauClient;

    connect(nouveauClient, SIGNAL(readyRead()), this, SLOT(donneesRecues()));
    connect(nouveauClient, SIGNAL(disconnected()), this, SLOT(deconnexionClient()));
}

void FenServeur::donneesRecues()
{
    // 1 : on reoit un paquet (ou un sous-paquet) d'un des clients

    // On dtermine quel client envoie le message (recherche du QTcpSocket du client)
    QTcpSocket *socket = qobject_cast<QTcpSocket *>(sender());
    if (socket == 0) // Si par hasard on n'a pas trouv le client  l'origine du signal, on arrte la mthode
        return;

    // Si tout va bien, on continue : on rcupre le message
    QDataStream in(socket);

    if (tailleMessage == 0) // Si on ne connat pas encore la taille du message, on essaie de la rcuprer
    {
        if (socket->bytesAvailable() < (int)sizeof(quint16)) // On n'a pas reu la taille du message en entier
             return;

        in >> tailleMessage; // Si on a reu la taille du message en entier, on la rcupre
    }

    // Si on connat la taille du message, on vrifie si on a reu le message en entier
    if (socket->bytesAvailable() < tailleMessage) // Si on n'a pas encore tout reu, on arrte la mthode
        return;


    // Si ces lignes s'excutent, c'est qu'on a reu tout le message : on peut le rcuprer !
    QString message;
    in >> message;


    // 2 : on renvoie le message  tous les clients
    envoyerATous(message);

    // 3 : remise de la taille du message  0 pour permettre la rception des futurs messages
    tailleMessage = 0;
}

void FenServeur::deconnexionClient()
{
    envoyerATous(tr("<em>Un client vient de se dconnecter</em>"));

    // On dtermine quel client se dconnecte
    QTcpSocket *socket = qobject_cast<QTcpSocket *>(sender());
    if (socket == 0) // Si par hasard on n'a pas trouv le client  l'origine du signal, on arrte la mthode
        return;

    clients.removeOne(socket);

    socket->deleteLater();
}

void FenServeur::envoyerATous(const QString &message)
{
    // Prparation du paquet
    QByteArray paquet;
    QDataStream out(&paquet, QIODevice::WriteOnly);

    out << (quint16) 0; // On crit 0 au dbut du paquet pour rserver la place pour crire la taille
    out << message; // On ajoute le message  la suite
    out.device()->seek(0); // On se replace au dbut du paquet
    out << (quint16) (paquet.size() - sizeof(quint16)); // On crase le 0 qu'on avait rserv par la longueur du message


    // Envoi du paquet prpar  tous les clients connects au serveur
    for (int i = 0; i < clients.size(); i++)
    {
        clients[i]->write(paquet);
    }

}
